/************************************************
rzb_db.c -- Razorback Database
Author -- Matthew Olney
************************************************/

#include <mysql.h>
#include <uuid/uuid.h>
#include <string.h>
#include <netinet/in.h>
#include <syslog.h>
#include <stddef.h>

#include "config.h"
#include "rzb_db.h"
#include "rzb_dispatcher.h"
#include "rzb_client.h"
#include "rzb_global.h"
#include "rzb_management_log.h"
#include "rzb_component_global.h"
#include "rzb_component_db.h"
#include "rzb_nug_manage.h"

typedef struct _MYSQL_CONNECTION
{
    MYSQL connection;
    struct _MYSQL_CONNECTION *next;
} MYSQLConnection;
static MYSQLConnection *freeConnectionList = NULL;
static int mysql_in_use = 0;
pthread_mutex_t mysql_mutex = PTHREAD_MUTEX_INITIALIZER;

static HRESULT connectToMysql(MYSQL *connection, const char *host, const char *user,
                              const char *password, unsigned port, const char *db_name,
                              unsigned flags)
{
    my_bool reconnect = 1;

    if (mysql_init(connection) == NULL)
    {
        perror("mysql_init failure");
        return R_FAIL;
    }

    if (mysql_real_connect(connection, host, user, password, db_name, port, NULL, flags) == NULL)
    {
        printError("mysql_real_connect failure", connection);
        mysql_close(connection);
        return R_FAIL;
    }
    // Automatically reconnect if the server drops us
    mysql_options(connection, MYSQL_OPT_RECONNECT, &reconnect);
#ifdef RZB_PERF
    pthread_mutex_lock(&perfmutex);
    perf.db_conn++;
    pthread_mutex_unlock(&perfmutex);
#endif /* RZB_PERF */
    return R_SUCCESS;
}

MYSQL *getMYSQLConnection(void)
{
    MYSQLConnection *connection;

    pthread_mutex_lock(&mysql_mutex);
    if (freeConnectionList)
    {
        connection = freeConnectionList;
        freeConnectionList = connection->next;
    }
    else if ((connection = calloc(1, sizeof(*connection))) == NULL ||
             (connectToMysql(&connection->connection, nrtconfig.dbhost,
                                                      nrtconfig.username, nrtconfig.password, 0,
                                                      nrtconfig.db, 0)) != R_SUCCESS)
    {
        printf("There are %d MYSQL connections still in use\n", mysql_in_use);
        pthread_mutex_unlock(&mysql_mutex);
        exit(-1);
    }
    mysql_in_use++;
    pthread_mutex_unlock(&mysql_mutex);
    return &connection->connection;
}

void putMYSQLConnection(MYSQL *conn)
{
    MYSQLConnection *connection = (MYSQLConnection *)conn;

    pthread_mutex_lock(&mysql_mutex);
    connection->next = freeConnectionList;
    freeConnectionList = connection;
    mysql_in_use--;
    pthread_mutex_unlock(&mysql_mutex);
}

void destroyMYSQLConnections(void)
{
    unsigned count = 0;
    MYSQLConnection *connection;

    pthread_mutex_lock(&mysql_mutex);
    printf("There are %d MYSQL connections still in use\n", mysql_in_use);
    while ((connection = freeConnectionList) != NULL)
    {
        count++;
        freeConnectionList = connection->next;
        mysql_close(&connection->connection);
        free(connection);
    }
    printf("Destroyed %u connections\n", count);
    pthread_mutex_unlock(&mysql_mutex);
}

HRESULT getEventID(unsigned *event_id)
{
    MYSQL *conn = getMYSQLConnection();

    if (mysql_query(conn, GET_NEW_EVENT_ID) != 0)
    {
        printError("Failed to get new event id", conn);
        putMYSQLConnection(conn);
        return R_FAIL;
    }
    *event_id = mysql_insert_id(conn);
    putMYSQLConnection(conn);
    return R_SUCCESS;
}

HRESULT insertComponentMetaData(METAPACKET *mpacket, const unsigned char *data, unsigned size)
{
    static const char INSERT_METADATA[] = "INSERT INTO metadata values (NULL, %u, '%.36s', now(), unhex('%.*s'))";
    MYSQL *conn;
    COMPONENT component;
    unsigned metabook_id;
    size_t request_size;
    char *request;
    char *protected_data;
    char type[SIZE_OF_UUID_STRING];

    uuid_unparse(CAP_DESCRIP, type);

    if ((checkComponent(mpacket->md5, mpacket->filesize, &component)) != R_SUCCESS)
    {
        submitLog(LOG_ERR, "Attempt to add data to an unknown md5/size", "insertMetaData");
        return R_FAIL;
    }

    if ((getMIDByCID(component.component_id, &metabook_id)) != R_SUCCESS)
    {
        submitLog(LOG_ERR, "Failed to get metabookID", "insertMetaData");
        return R_FAIL;
    }

    if ((protectHex(data, size, &protected_data)) != R_SUCCESS)
    {
        submitLog(LOG_ERR, "Unable to protect meta data", "inseretMetaData");
        return R_FAIL;
    }

    request_size = (size * 2) + SIZE_OF_UUID_STRING + sizeof(INSERT_METADATA) + MAX_INT_STRING_SIZE;
    if ((request = malloc(request_size)) == NULL)
    {
        submitLog(LOG_ERR, "Unable to allocate space for metadata", "insertMetaData");
        return R_FAIL;
    }

    snprintf(request, request_size, INSERT_METADATA, metabook_id, type, size*2, protected_data);

    conn = getMYSQLConnection();
    if (mysql_query(conn, request) != 0)
    {
        submitLog(LOG_ERR, "Failed to execute query", "insertComponentMetaData");
        putMYSQLConnection(conn);
        free(request);
        return R_DB_FAIL;
    }

    putMYSQLConnection(conn);
    free(request);
    return R_SUCCESS;
}

/* Get Metabook ID by Component ID */
HRESULT getMIDByCID(unsigned component_id, unsigned *metabook_id)
{
    MYSQL *conn;
    MYSQL_RES *response;
    MYSQL_ROW row;
    char request[256];
    unsigned m_id;

    snprintf(request, sizeof(request), GET_MID_CID, component_id);

    conn = getMYSQLConnection();
    /* Execute the query */
    if (mysql_query(conn, request) != 0)
    {
        submitLog(LOG_ERR, "Failed to get metabook_id", "getMIDByCID");
        putMYSQLConnection(conn);
        return R_DB_FAIL;
    }

    response = mysql_store_result(conn);

    /* Check return for valid data */
    if (!response)
    {
        submitLog(LOG_ERR, "Invalid response object", "getMIDByCID");
        putMYSQLConnection(conn);
        return R_FAIL;
    }

    row = mysql_fetch_row(response);
    if (!row || !row[0] || !*row[0])
    {
        submitLog(LOG_ERR, "Row return was NULL", "getMIDByCID");
        mysql_free_result(response);
        putMYSQLConnection(conn);
        return R_FAIL;
    }

    m_id = (unsigned)strtoul(row[0], NULL, 10);
    mysql_free_result(response);
    putMYSQLConnection(conn);

    if (!m_id)
    {
        if (createMetaBook(&m_id) != R_SUCCESS)
        {
            submitLog(LOG_ERR, "Unable to create metabook", "getMIDByCID");
            return R_FAIL;
        }
        if (insertMIDByCID(component_id, m_id) != R_SUCCESS)
        {
            submitLog(LOG_ERR, "Unabled to attach metabook id to component", "getMIDByCID");
            return R_FAIL;
        }

        *metabook_id = m_id;
        return R_SUCCESS;
    }
    *metabook_id = (unsigned)strtoul(row[0], NULL, 10);
    return R_SUCCESS;
}

HRESULT insertMIDByCID(unsigned component_id, unsigned metabook_id)
{
    MYSQL *conn = getMYSQLConnection();
    char request[256];

    snprintf(request, sizeof(request), UPDATE_CID_MID, metabook_id, component_id);

    /* Execute the query */
    if (mysql_query(conn, request) != 0)
    {
        submitLog(LOG_ERR, "Failed to execute query", "metabook_id");
        putMYSQLConnection(conn);
        return R_DB_FAIL;
    }

    putMYSQLConnection(conn);
    return R_SUCCESS;
}


/* Get Metabook ID by Component ID */
HRESULT createMetaBook(unsigned *metabook)
{
    MYSQL *conn = getMYSQLConnection();

    /* Execute the query */
    if (mysql_query (conn, CREATE_METABOOK) != 0)
    {
        submitLog(LOG_ERR, "Failed to execute query", "createMetaBook");
        putMYSQLConnection(conn);
        return R_DB_FAIL;
    }

    *metabook = mysql_insert_id(conn);

    putMYSQLConnection(conn);
    return R_SUCCESS;
}



HRESULT buildUUIDList(unsigned *count, uuid_t **newList, const char *request)
{
    MYSQL *conn = getMYSQLConnection();
    MYSQL_RES *UUIDList;
    MYSQL_ROW row;

    unsigned i;
    unsigned numUUIDs;

    /* Execute supplied query */
    if (mysql_query(conn, request) != 0)
    {
        printError("Failed to send request", conn);
        putMYSQLConnection(conn);
        return R_DB_FAIL;
    }

    UUIDList = mysql_store_result(conn);

    if (!UUIDList)
    {
        if (mysql_field_count(conn) == 0)
        {
            printError("Field count is zero for list of UUIDs", conn);
            putMYSQLConnection(conn);
            return R_FAIL;
        }
        else
        {
            printError("Failed to get list of UUIDs", conn);
            putMYSQLConnection(conn);
            return R_DB_FAIL;
        }
    }

    numUUIDs = mysql_num_rows(UUIDList);

    if ((*newList = malloc(numUUIDs * sizeof(uuid_t))) == NULL)
    {
        perror("Failed to allocate space for new list\n");
        mysql_free_result(UUIDList);
        putMYSQLConnection(conn);
        return R_MALLOC_FAIL;
    }

    /* Build the list */
    for (i = 0; i < numUUIDs; i++)
    {
        row = mysql_fetch_row(UUIDList);
        if (row[0] == NULL)
            continue;
        uuid_parse(row[0], (*newList)[i]);
    }
    /* List completed */
    *count = numUUIDs;

    /* Cleanup mysql query */
    mysql_free_result(UUIDList);
    putMYSQLConnection(conn);

    return R_SUCCESS;
}

HRESULT printUUIDList(uuid_t *list, unsigned listSize)
{
    unsigned i;
    char uuid_str[SIZE_OF_UUID_STRING];
    char description[MAX_DESCRIPTION_SIZE];

    for (i = 0; i < listSize; i++)
    {
        uuid_unparse(list[i], uuid_str);
        getUUIDDesc (uuid_str, description);
        printf("%s\t%s\n", uuid_str, description);
    }

    return R_SUCCESS;
}

HRESULT getUUIDDesc(const char *UUID, char *description)
{
    MYSQL *conn;
    MYSQL_RES *UUID_Desc;
    MYSQL_ROW row;
    char *requestFinal;
    size_t requestSize = sizeof(RETRIEVE_DESCRIPTION_SQL) + strlen(UUID);

    snprintf(description, MAX_DESCRIPTION_SIZE, "%s", "Unknown");
    if ((requestFinal = malloc(requestSize)) == NULL)
        return R_MALLOC_FAIL;

    snprintf(requestFinal, requestSize, RETRIEVE_DESCRIPTION_SQL, UUID);

    if ((conn = getMYSQLConnection()) == NULL)
    {
        free(requestFinal);
        return R_FAIL;
    }

    /* Execute the query */
    if (mysql_query(conn, requestFinal) != 0)
    {
        printError("Failed initial results check", conn);
        free(requestFinal);
        putMYSQLConnection(conn);
        return R_DB_FAIL;
    }
    free(requestFinal);

    UUID_Desc = mysql_store_result(conn);

    /* Check return for valid data */
    if (!UUID_Desc)
    {
        if (mysql_field_count(conn) == 0)
        {
            putMYSQLConnection(conn);
            return R_NOT_FOUND;
        }
        else
        {
            printError("Failed to get list of valid nuggets", conn);
            putMYSQLConnection(conn);
            return R_DB_FAIL;
        }
    }

    row = mysql_fetch_row(UUID_Desc);

    if (!row || !row[0] || !*row[0])
    {
        mysql_free_result(UUID_Desc);
        putMYSQLConnection(conn);
        return R_DB_FAIL;
    }
    snprintf(description, MAX_DESCRIPTION_SIZE, "%s", row[0]);

    mysql_free_result(UUID_Desc);

    putMYSQLConnection(conn);
    return R_SUCCESS;
}

static HRESULT verifyUID(NUGGETENTRY *newNug)
{
    static const char VERIFY_UUID_SQL[] = "SELECT * from nuggets where Nugget_ID = '%u'";
    NUGGETENTRY *target = newNug;
    MYSQL *conn;
    MYSQL_RES *nugEntry;
    MYSQL_ROW row;
    char *request;
    char uuid_str[SIZE_OF_UUID_STRING];
    size_t requestSize = sizeof(VERIFY_UUID_SQL) + MAX_INT_STRING_SIZE;
    struct in_addr origIP, newIP;
    unsigned short origPort, newPort;

    if ((request = malloc(requestSize)) == NULL)
        return R_MALLOC_FAIL;

    /* SELECT * from NUGGETS where Nugget_ID = 'x' */
    snprintf(request, requestSize, VERIFY_UUID_SQL, target->nuggetid);

    conn = getMYSQLConnection();
    if (mysql_query(conn, request) != 0)
    {
        printError("Failed to verify UID\n", conn);
        free(request);
        putMYSQLConnection(conn);
        return R_DB_FAIL;
    }

    free(request);

    if ((nugEntry = mysql_store_result(conn)) == NULL)
    {
        printError("Failed to retrieve results for Verify UID\n", conn);
        mysql_free_result(nugEntry);
        putMYSQLConnection(conn);
        return R_FAIL;
    }

    if ((mysql_num_rows(nugEntry)) == 0)
    {
        mysql_free_result(nugEntry);
        putMYSQLConnection(conn);
        return R_SUCCESS;
    }

    row = mysql_fetch_row(nugEntry);

    origIP.s_addr = (unsigned)strtoul(row[5], NULL, 10);
    newIP = target->nugaddr;
    origPort = (unsigned short)strtoul(row[6], NULL, 10);
    newPort = target->nugport;

    uuid_unparse(target->nugapp, uuid_str);

    if ((origIP.s_addr == newIP.s_addr) && (origPort == newPort) && strcmp(row[2], uuid_str) == 0)
    {
        mysql_free_result(nugEntry);
        putMYSQLConnection(conn);
        return R_FOUND;
    }
    mysql_free_result(nugEntry);
    putMYSQLConnection(conn);
    return R_FAIL;
}

static HRESULT reassignUID(NUGGETENTRY *newNug)
{
    NUGGETENTRY *target = newNug;
    MYSQL *conn;
    MYSQL_RES *nugEntry;
    MYSQL_ROW row;
    char *request;
    size_t requestSize = sizeof(FIND_PREREG_SQL) + 2 * MAX_INT_STRING_SIZE;

    if ((request = malloc(requestSize)) == NULL)
    {
        perror("Failed to malloc for reassign UID requst\n");
        return R_MALLOC_FAIL;
    }

    snprintf(request, requestSize, FIND_PREREG_SQL, target->nugaddr.s_addr, target->nugport);

    conn = getMYSQLConnection();

    if (mysql_query(conn, request) != 0)
    {
        free(request);
        printf("Request: %s\n", request);
        printError("Failed SQL in Reassign UID\n", conn);
        return R_DB_FAIL;
    }
    free(request);

    if ((nugEntry = mysql_store_result(conn)) == NULL)
    {
        printError("Failed to retrieve results for Reassign UID\n", conn);
        putMYSQLConnection(conn);
        return R_MALLOC_FAIL;
    }

    if ((mysql_num_rows(nugEntry)) == 0)
    {
        mysql_free_result(nugEntry);
        putMYSQLConnection(conn);
        return R_FAIL;
    }

    row = mysql_fetch_row(nugEntry);
    target->nuggetid = strtoul(row[0], NULL, 10);
    mysql_free_result(nugEntry);

    putMYSQLConnection(conn);
    return R_SUCCESS;
}

static HRESULT dbRegisterWithNugID(NUGGETENTRY *newNug)
{
    static const char REG_ENTRY_SQL[] = "INSERT INTO nuggets values ('%u', '%.36s', '%.36s', '%u', '%s', '%u', '%u', NULL, NULL)";
    MYSQL *conn = getMYSQLConnection();
    NUGGETENTRY *target = newNug;
    char request[MAX_REG_SIZE];
    char nugtype_str[SIZE_OF_UUID_STRING];
    char nugapp_str[SIZE_OF_UUID_STRING];

    uuid_unparse(target->nugtype, nugtype_str);
    uuid_unparse(target->nugapp, nugapp_str);

    snprintf(request, sizeof(request), REG_ENTRY_SQL,
             target->nuggetid,
             nugtype_str,
             nugapp_str,
             target->freeinstances,
             target->name,
             target->nugaddr.s_addr,
             target->nugport);

    if (mysql_query(conn, request) != 0)
    {
        printf("DbRegisterWithNugID Request: %s\n", request);
        printError("Failed to add nugget\n", conn);
        putMYSQLConnection(conn);
        return R_DB_FAIL;
    }

    putMYSQLConnection(conn);
    return R_SUCCESS;
}

HRESULT dbRegisterNugget(NUGGETENTRY *newNug)
{
    static const char REG_NEW_ENTRY_SQL[] = "INSERT INTO nuggets values ('NULL', '%.36s', '%.36s', '%u', '%s', '%u', '%u', NULL, NULL)";
    NUGGETENTRY *target = newNug;
    MYSQL *conn;
    MYSQL_RES *newNugID;
    MYSQL_ROW row;
    char request[MAX_REG_SIZE];
    HRESULT out;
    char nugtype_str[SIZE_OF_UUID_STRING];
    char nugapp_str[SIZE_OF_UUID_STRING];

    /* Is the nugget trying to declare a Nugget ID to use? */
    if (target->nuggetid != 0)
    {
        out = verifyUID(target);    /* Is that Nugget ID used by someone else? */
        if (out == R_FOUND)          /* R_FOUND means the nugget ID belongs to this nugget */
        {
            printf("Reusing previous UID of %u\n", target->nuggetid);
            return R_SUCCESS;
        }
        else if (out == R_SUCCESS)
        {
            /* R_SUCCESS means the id is unallocated and may be used */
            /* Call dbRegisterWithNugID to claim the ID */
            dbRegisterWithNugID(target);
            return R_SUCCESS;
        }
    }

    /* Has this nugget already registered before, if so use its previous id */
    if (reassignUID(target) == R_SUCCESS)
        return R_SUCCESS;

    uuid_unparse(target->nugtype, nugtype_str);
    uuid_unparse(target->nugapp, nugapp_str);

    snprintf(request, sizeof(request), REG_NEW_ENTRY_SQL,
             nugtype_str,
             nugapp_str,
             target->freeinstances,
             target->name,
             target->nugaddr.s_addr,
             target->nugport);
    conn = getMYSQLConnection();
    if (mysql_query(conn, request) != 0)
    {
        printf("dbRegisterNugget Request: %s\n", request);
        printError("Failed to add nugget\n", conn);
        putMYSQLConnection(conn);
        return R_ADD_FAIL;
    }

    /* Retrieve the UID for the nugget */
    if (mysql_query(conn, RETRIEVE_LAST_NUGID) !=0)
    {
        printError("Failed to retrieve new nuggetid\n", conn);
        putMYSQLConnection(conn);
        return R_MALLOC_FAIL;
    }

    if ((newNugID = mysql_store_result(conn)) == NULL)
    {
        putMYSQLConnection(conn);
        return R_DB_FAIL;
    }

    row = mysql_fetch_row(newNugID);

    if (!row || row[0] == NULL)
    {
        mysql_free_result(newNugID);
        putMYSQLConnection(conn);
        return R_DB_FAIL;
    }

    target->nuggetid = strtoul(row[0], NULL, 10);

    mysql_free_result(newNugID);
    putMYSQLConnection(conn);
    return R_SUCCESS;
}

HRESULT checkMD5(const unsigned char *md5, unsigned size, int *cache_result)
{
    static const char CHECK_MD5_SQL[] = "SELECT cast(Threat_flags as unsigned) FROM components WHERE crc_id=CRC32(CONCAT(UPPER(\"%32s\"), CAST(%u AS CHAR))) AND size = '%d' AND Checksum = '%32s'";
    MYSQL *conn;
    MYSQL_RES *Threat_flag;
    MYSQL_ROW row;

    char md5string[SAFE_MD5_SIZE];
    char *request;
    size_t request_size;

    snprintf(md5string, sizeof(md5string), "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
             md5[0], md5[1], md5[2], md5[3], md5[4], md5[5], md5[6], md5[7],
             md5[8], md5[9], md5[10], md5[11], md5[12], md5[13], md5[14], md5[15]);

    request_size = sizeof(CHECK_MD5_SQL) + strlen(md5string) * 2 + MAX_INT_STRING_SIZE * 2;
    if ((request = malloc(request_size)) == NULL)
        return R_MALLOC_FAIL;

    snprintf(request, request_size, CHECK_MD5_SQL, md5string, size, size, md5string);

    conn = getMYSQLConnection();
    if (!conn)
        return R_FAIL;

    if (mysql_query(conn, request) !=0)
    {
        printError("Failed results check in checkMD5\n", conn);
        free(request);
        putMYSQLConnection(conn);
        return R_DB_FAIL;
    }

    free(request);

    Threat_flag = mysql_store_result(conn);

    if (!Threat_flag)
    {
        if (mysql_field_count(conn) == 0)
        {
            printf("Missed!\n");
#ifdef RZB_PERF
            pthread_mutex_lock(&perfmutex);
            perf.db_cache_miss++;
            pthread_mutex_unlock(&perfmutex);
#endif /* RZB_PERF */
            putMYSQLConnection(conn);
            return R_NOT_FOUND;
        }
        else
        {
            printError("Failed to get results in checkMD5\n", conn);
            putMYSQLConnection(conn);
            return R_DB_FAIL;
        }
    }

    if ((row = mysql_fetch_row(Threat_flag)) == NULL)
    {
#ifdef RZB_PERF
        pthread_mutex_lock(&perfmutex);
        perf.db_cache_miss++;
        pthread_mutex_unlock(&perfmutex);
#endif /* RZB_PERF */
        mysql_free_result(Threat_flag);
        putMYSQLConnection(conn);
        return R_NOT_FOUND;
    }

    if (row[0] == NULL)
    {
        printError("Row 0 is null in checkMD5\n", conn);
        mysql_free_result(Threat_flag);
        putMYSQLConnection(conn);
        return R_DB_FAIL;
    }

    if (strtoul(row[0], NULL, 10) & KNOWN_BAD)
    {
#ifdef RZB_PERF
        pthread_mutex_lock(&perfmutex);
        perf.db_cache_hit_bad++;
        pthread_mutex_unlock(&perfmutex);
#endif /* RZB_PERF */
        *cache_result = CACHE_HIT_KNOWN_BAD;
        mysql_free_result(Threat_flag);
        putMYSQLConnection(conn);
        return R_SUCCESS;
    }

    if (strtoul(row[0], NULL, 10) & KNOWN_GOOD)
    {
#ifdef RZB_PERF
        pthread_mutex_lock(&perfmutex);
        perf.db_cache_hit_good++;
        pthread_mutex_unlock(&perfmutex);
#endif /* RZB_PERF */
        *cache_result = CACHE_HIT_KNOWN_GOOD;
        mysql_free_result(Threat_flag);
        putMYSQLConnection(conn);
        return R_SUCCESS;
    }

#ifdef RZB_PERF
    pthread_mutex_lock(&perfmutex);
    perf.db_cache_miss++;
    pthread_mutex_unlock(&perfmutex);
#endif /* RZB_PERF */
    mysql_free_result(Threat_flag);
    putMYSQLConnection(conn);
    return R_NOT_FOUND;
}

/*
 * IN:            *data, size, **protected_data
 * OUT:           HRESULT, population of protected_data with ptr to normalized data
 * FUNCTION:      Ensure tha any dangerous characters are normalized prior to
 *                databse query being executed.
*/
HRESULT protectString(const char *data, unsigned size, char **protected_data)
{
    MYSQL *conn;
    char *p_data;
    char *sc_check;
    unsigned p_size;
    unsigned sc_count = 0;
    unsigned sc_ptr = 0;
    unsigned i;

    if (data == NULL || size == 0)
    {
        *protected_data = NULL;
        return R_SUCCESS;
    }

    if ((conn = getMYSQLConnection()) == NULL)
        return R_FAIL;

    /* OK, mysql_real_escape_string doesn't handle semi-colons or backslashes */
    /* We'll run through the strings and count them and then allocate space to */
    /* handle them if necessary */

    // Count all the semicolons and backslashes
    for (i=0; i < size; i++)
    {
        if (data[i] == ';' || data[i] == '\\')
            sc_count++;
    }

    // Run this only if we have to
    if (sc_count)
    {
        if ((sc_check = malloc(size + sc_count)) == NULL)
        {
            putMYSQLConnection(conn);
            return R_MALLOC_FAIL;
        }
        i = 0;
        while (sc_ptr != sc_count)
        {
            // If we find a semicolon or backslash, insert a backslash
            if (data[i] == ';' || data[i] == '\\')
            {
                sc_check[i+sc_ptr] = '\x5C';
                sc_ptr++;
            }
            sc_check[i+sc_ptr] = data[i];
            i++;
        }
        p_size = (2 * (size + sc_count)) + 1;
        p_data = malloc(p_size);

        if (p_data == NULL)
        {
            submitLog(LOG_ERR, "Failed to allocate space for p_data", "protectString");
            putMYSQLConnection(conn);
            return R_MALLOC_FAIL;
        }
        mysql_real_escape_string(conn, p_data, sc_check, size + sc_count);
        free(sc_check);
    }
    else
    {
        p_size = (2 * size) +1;

        if ((p_data = malloc(p_size)) == NULL)
        {
            submitLog(LOG_ERR, "Failed to allocate space for p_data", "protectString");
            putMYSQLConnection(conn);
            return R_MALLOC_FAIL;
        }
        mysql_real_escape_string(conn, p_data, data, size);
    }

    *protected_data = p_data;
    putMYSQLConnection(conn);
    return R_SUCCESS;
}

/*
 * IN:            *data, size, **protected_data
 * OUT:           HRESULT, population of protected_data with ptr to normalized data
 * FUNCTION:      For blocks of binary data, ascii hex encode them to ensure database
 *                query does not fail.
*/
HRESULT protectHex(const unsigned char *data, unsigned size, char **protected_data)
{
    char *p_data;

    if ((p_data = malloc((2 * size) + 1)) == NULL)
        return R_MALLOC_FAIL;

    mysql_hex_string(p_data, (const char *)data, size);
    *protected_data = p_data;
    return R_SUCCESS;
}

HRESULT insertLog(unsigned level, const char *msg, const char *src)
{
    MYSQL *conn;
    char *request;
    size_t request_size;

    request_size = sizeof(INSERT_LOG_ENTRY) + MAX_INT_STRING_SIZE;
    request_size += strlen(msg);
    request_size += strlen(src);

    if ((request = malloc(request_size)) == NULL)
    {
        perror("Failed to allocate memory for request in regwithnug\n");
        return R_MALLOC_FAIL;
    }

    snprintf(request, request_size, INSERT_LOG_ENTRY, src, msg, level);

    if ((conn = getMYSQLConnection()) == NULL)
    {
        free(request);
        return R_FAIL;
    }
    if (mysql_query(conn, request) != 0)
    {
        printf("insertLog Request: %s\n", request);
        printError("Failed to insert log", conn);
        free(request);
        putMYSQLConnection(conn);
        return R_DB_FAIL;
    }

    free(request);
    putMYSQLConnection(conn);
    return R_SUCCESS;
}

void printError(const char * errMessage, MYSQL *conn)
{
    unsigned myerrno = mysql_errno(conn);
    const char *mystate = mysql_sqlstate(conn);
    const char *myerror = mysql_error(conn);
    printf("%s: Error %u (%s): %s\n", errMessage, myerrno, mystate, myerror);
}

